﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;

namespace TestProject;

/// <summary>
/// 알림 헬퍼
/// </summary>
public class NotificationHelper
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 알림 컨트롤 큐
    /// </summary>
    private readonly Queue<NotificationControl> notificationControlQueue = new Queue<NotificationControl>();

    /// <summary>
    /// 컨테이너 패널
    /// </summary>
    private readonly Panel containerPanel;

    /// <summary>
    /// 현재 알림 컨트롤
    /// </summary>
    private NotificationControl currentNotificationControl;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - NotificationHelper(containerPanel)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="containerPanel">컨테이너 패널</param>
    public NotificationHelper(Panel containerPanel)
    {
        this.containerPanel = containerPanel;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 알림 표시하기 - ShowNotification(notificationControl)

    /// <summary>
    /// 알림 표시하기
    /// </summary>
    /// <param name="notificationControl"></param>
    public void ShowNotification(NotificationControl notificationControl)
    {
        notificationControl.Closed += notificationControl_Closed;

        if(this.currentNotificationControl == null)
        {
            ShowNextNotification(notificationControl);
        }
        else
        {
            this.notificationControlQueue.Enqueue(notificationControl);
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region 알림 컨트롤 닫은 경우 처리하기 - notificationControl_Closed(sender, e)

    /// <summary>
    /// 알림 컨트롤 닫은 경우 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void notificationControl_Closed(object sender, EventArgs e)
    {
        this.currentNotificationControl = null;

        if(this.notificationControlQueue.Count > 0)
        {
            ShowNextNotification(this.notificationControlQueue.Dequeue());
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region 다음 알림 표시하기 - ShowNextNotification(notificationControl)

    /// <summary>
    /// 다음 알림 표시하기
    /// </summary>
    /// <param name="notificationControl">알림 컨트롤</param>
    private void ShowNextNotification(NotificationControl notificationControl)
    {
        this.currentNotificationControl = notificationControl;

        this.containerPanel.Children.Add(notificationControl);
    }

    #endregion
}